SystemOrganization addCategory: #OSProcessIO!
SystemOrganization addCategory: #'OSProcessIO-Utils'!

Trait named: #TOSProcessParsedIO
	uses: {}
	category: 'OSProcessIO-Utils'!

!TOSProcessParsedIO classSide methodsFor: 'execution' stamp: 'SR 9/1/2007 14:34'!
model: aModel command: aSelector parser: aBlock
	^ super new
		model: aModel;
		command: aSelector;
		parser: aBlock;
		result! !

!TOSProcessParsedIO methodsFor: 'private' stamp: 'SR 1/7/2009 11:07'!
basicResult
	^ self parser valueWithPossibleArgument: self! !

!TOSProcessParsedIO methodsFor: 'accessing-dynamic' stamp: 'SR 1/7/2009 11:10'!
result
	" throw an exception if the command was not found in the current environment, otherwise return the result "
	super result.
	(self errorString includesSubstring: 'command not found' caseSensitive: false)
		ifTrue: [ ^ OSProcessCommandNotFound signal ].
	^ self basicResult! !

Trait named: #TOSProcessStreamedIO
	uses: {}
	category: 'OSProcessIO-Utils'!

!TOSProcessStreamedIO methodsFor: 'private-exceptions' stamp: 'SR 8/22/2007 20:02'!
assertStreamConstruction: aFile
	(FileStream isAFileNamed: aFile) ifFalse: [ self exceptionClassStreamConstruction signal ]! !

!TOSProcessStreamedIO methodsFor: 'private-exceptions' stamp: 'SR 8/22/2007 20:02'!
assertStreamDestruction: aFile
	(FileStream isAFileNamed: aFile) ifTrue: [ self exceptionClassStreamDestruction signal ]! !

!TOSProcessStreamedIO methodsFor: 'private-accessing' stamp: 'SR 8/22/2007 20:09'!
defaultStreamIdentifier
	^ 'osprocessio'! !

!TOSProcessStreamedIO methodsFor: 'private-utils' stamp: 'SR 8/22/2007 20:01'!
destroyStream: aString
	FileDirectory deleteFilePath: aString.
	FileDirectory default deleteFileNamed: aString.
	self assertStreamDestruction: aString! !

!TOSProcessStreamedIO methodsFor: 'private' stamp: 'SR 8/22/2007 19:57'!
destroyStreams
	self destroyStream: self outputStreamName.
	self destroyStream: self errorStreamName.! !

!TOSProcessStreamedIO methodsFor: 'private-exceptions' stamp: 'SR 8/22/2007 20:02'!
exceptionClassStreamConstruction
	^ OSProcessStreamConstructionFailed! !

!TOSProcessStreamedIO methodsFor: 'private-exceptions' stamp: 'SR 8/22/2007 20:02'!
exceptionClassStreamDestruction
	^ OSProcessStreamDestructionFailed! !

!TOSProcessStreamedIO methodsFor: 'accessing-dynamic' stamp: 'SR 8/22/2007 19:55'!
execute
	" we simulate stdout and stderr "
	[ self makeStreams; basicExecute; readStreams ] 
		ensure: [ self destroyStreams ]! !

!TOSProcessStreamedIO methodsFor: 'testing' stamp: 'SR 8/22/2007 20:00'!
executed
	^ super executed
		and: [ self outputString isNil not or: [ self errorString isNil not ] ]! !

!TOSProcessStreamedIO methodsFor: 'private-accessing' stamp: 'SR 8/22/2007 20:03'!
ioCommand
	^ super ioCommand, self ioStreamCommands! !

!TOSProcessStreamedIO methodsFor: 'private-accessing' stamp: 'SR 8/22/2007 20:03'!
ioErrorStreamCommand
	^ self ioStreamCommandFor: 2 with: self errorStreamName! !

!TOSProcessStreamedIO methodsFor: 'private-accessing' stamp: 'SR 8/22/2007 20:03'!
ioOutputStreamCommand
	^ self ioStreamCommandFor: 1 with: self outputStreamName! !

!TOSProcessStreamedIO methodsFor: 'private-utils' stamp: 'SR 9/9/2007 12:15'!
ioStreamCommandFor: anInteger with: aString
	^ ' ', anInteger asString, '>', '"', aString, '"', ' '! !

!TOSProcessStreamedIO methodsFor: 'private-accessing' stamp: 'SR 8/22/2007 20:47'!
ioStreamCommands
	^  self ioOutputStreamCommand, self ioErrorStreamCommand! !

!TOSProcessStreamedIO methodsFor: 'private-accessing' stamp: 'SR 8/22/2007 20:03'!
ioStreamName
	^ self ioStreamTarget, '.', self ioStreamIdentifier! !

!TOSProcessStreamedIO methodsFor: 'private-utils' stamp: 'SR 8/22/2007 20:37'!
makeStream: aString
	| stream |
	stream := StandardFileStream forceNewFileNamed: aString.
	self assertStreamConstruction: aString.
	stream close.
	^ stream
	! !

!TOSProcessStreamedIO methodsFor: 'private' stamp: 'SR 8/22/2007 19:57'!
makeStreams
	self makeStream: self errorStreamName.
	self makeStream: self outputStreamName! !

!TOSProcessStreamedIO methodsFor: 'accessing-dynamic' stamp: 'SR 6/2/2008 10:37'!
messageText
	^ String streamContents: [ :stream | stream
		nextPutAll: 'ERROR:'; cr;
		nextPutAll: self errorString; cr; cr;
		nextPutAll: 'OUTPUT:'; cr;
		nextPutAll: self outputString ]! !

!TOSProcessStreamedIO methodsFor: 'private' stamp: 'SR 8/22/2007 19:58'!
primitiveExecute
	^ OSProcess waitForCommand: self ioCommand! !

!TOSProcessStreamedIO methodsFor: 'private-utils' stamp: 'SR 8/22/2007 20:37'!
readStream: aString
	| stream contents |
	stream := StandardFileStream fileNamed: aString.
	contents := stream contentsOfEntireFile.
	stream close.
	^ contents
	! !

!TOSProcessStreamedIO methodsFor: 'private' stamp: 'SR 8/22/2007 19:57'!
readStreams
	self readOutputStream.
	self readErrorStream.! !

!TOSProcessStreamedIO methodsFor: 'testing' stamp: 'SR 8/22/2007 20:00'!
succeeded
	^ super succeeded
		or: [ self errorString isNil ]! !

!String methodsFor: '*OSProcessIO' stamp: 'SR 9/1/2007 09:19'!
command
	^ self! !

Error subclass: #OSProcessFailed
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OSProcessIO-Utils'!

OSProcessFailed subclass: #OSProcessCommandNotFound
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OSProcessIO-Utils'!

OSProcessFailed subclass: #OSProcessStreamConstructionFailed
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OSProcessIO-Utils'!

OSProcessFailed subclass: #OSProcessStreamDestructionFailed
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OSProcessIO-Utils'!

Object subclass: #OSProcessIO
	instanceVariableNames: 'process'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OSProcessIO-Utils'!

!OSProcessIO commentStamp: 'SR 8/22/2007 21:08' prior: 0!
this is a generic superclass for all IO processes, use one of the subclasses for process io!

!OSProcessIO class methodsFor: 'accessing-dynamic' stamp: 'SR 9/1/2007 09:17'!
executableFor: anObject
	^ OSProcess thisOSProcess executableFor: anObject! !

!OSProcessIO methodsFor: 'private' stamp: 'SR 8/22/2007 21:03'!
absoluteUrlFor: aString
	^ FileDirectory default fullName, FileDirectory pathNameDelimiter asString, aString! !

!OSProcessIO methodsFor: 'private-exceptions' stamp: 'SR 8/22/2007 19:48'!
assertProcess: aProcess
	aProcess succeeded ifFalse: [ self exceptionClassProcess signal ]! !

!OSProcessIO methodsFor: 'private' stamp: 'SR 8/22/2007 19:52'!
basicExecute
	process := OSProcess waitForCommand: self ioCommand.
	self assertProcess: process! !

!OSProcessIO methodsFor: 'private-exceptions' stamp: 'SR 8/22/2007 19:48'!
exceptionClassProcess
	^ OSProcessFailed! !

!OSProcessIO methodsFor: 'accessing-dynamic' stamp: 'SR 8/22/2007 19:48'!
execute
	self basicExecute! !

!OSProcessIO methodsFor: 'testing' stamp: 'SR 8/22/2007 19:48'!
executed
	^ self process isNil not 
		and: [ process isComplete ]! !

!OSProcessIO methodsFor: 'accessing-dynamic' stamp: 'SR 6/2/2008 10:08'!
messageText
	^ self asString! !

!OSProcessIO methodsFor: 'printing' stamp: 'SR 8/22/2007 20:18'!
printOn: aStream
	aStream nextPutAll: self ioCommand! !

!OSProcessIO methodsFor: 'accessing' stamp: 'SR 8/22/2007 19:46'!
process
	^ process! !

!OSProcessIO methodsFor: 'private' stamp: 'SR 8/22/2007 19:51'!
resetCache
	process := nil! !

!OSProcessIO methodsFor: 'accessing-dynamic' stamp: 'SR 8/22/2007 19:47'!
result
	self executed ifFalse: [ self execute ].
	^ self! !

!OSProcessIO methodsFor: 'testing' stamp: 'SR 8/22/2007 19:48'!
succeeded
	^ self executed 
		and: [ self process succeeded ]! !

OSProcessIO subclass: #OSProcessRawIO
	instanceVariableNames: 'command'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OSProcessIO'!

!OSProcessRawIO commentStamp: 'SR 8/22/2007 21:09' prior: 0!
REQUIRED PARAMETERS:
- command = a command string (example: 'open -a safari')!

!OSProcessRawIO class methodsFor: 'execution' stamp: 'SR 8/22/2007 19:51'!
command: aString
	^ super new
		command: aString;
		execute! !

!OSProcessRawIO methodsFor: 'accessing' stamp: 'SR 8/22/2007 19:50'!
command
	^ command! !

!OSProcessRawIO methodsFor: 'accessing' stamp: 'SR 8/22/2007 19:50'!
command: aSelector
	self resetCache.
	command := aSelector! !

!OSProcessRawIO methodsFor: 'private' stamp: 'SR 8/22/2007 19:52'!
ioCommand
	^ self command! !

OSProcessRawIO subclass: #OSProcessStreamedRawIO
	uses: TOSProcessStreamedIO
	instanceVariableNames: 'error output'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OSProcessIO'!

!OSProcessStreamedRawIO commentStamp: 'SR 8/22/2007 21:10' prior: 0!
same as above but it will collect the output and error from the shell by default!

OSProcessStreamedRawIO class
	uses: TOSProcessStreamedIO classTrait
	instanceVariableNames: ''!

OSProcessStreamedRawIO subclass: #OSProcessParsedRawIO
	uses: TOSProcessParsedIO
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OSProcessIO'!

OSProcessParsedRawIO class
	uses: TOSProcessParsedIO classTrait
	instanceVariableNames: ''!

!OSProcessParsedRawIO methodsFor: 'accessing' stamp: 'SR 9/1/2007 14:32'!
parser
	^ parser! !

!OSProcessParsedRawIO methodsFor: 'accessing' stamp: 'SR 9/1/2007 14:39'!
parser: aBlock
	parser := aBlock! !

!OSProcessStreamedRawIO methodsFor: 'private' stamp: 'SR 8/22/2007 19:58'!
basicExecute
	process := self primitiveExecute
	" we do not assert this kind of process - instead we have a result and error log "! !

!OSProcessStreamedRawIO methodsFor: 'private-accessing' stamp: 'SR 8/22/2007 20:59'!
errorStreamName
	^  self absoluteUrlFor: (self ioStreamName, '.error')! !

!OSProcessStreamedRawIO methodsFor: 'accessing' stamp: 'SR 8/22/2007 19:54'!
errorString
	^ error! !

!OSProcessStreamedRawIO methodsFor: 'private-accessing' stamp: 'SR 8/22/2007 20:08'!
ioStreamIdentifier
	^ self defaultStreamIdentifier! !

!OSProcessStreamedRawIO methodsFor: 'private-accessing' stamp: 'SR 8/22/2007 20:11'!
ioStreamTarget
	^ self command hash asString! !

!OSProcessStreamedRawIO methodsFor: 'private-accessing' stamp: 'SR 8/22/2007 20:59'!
outputStreamName
	^ self absoluteUrlFor: (self ioStreamName, '.output')! !

!OSProcessStreamedRawIO methodsFor: 'accessing' stamp: 'SR 8/22/2007 19:54'!
outputString
	^ output! !

!OSProcessStreamedRawIO methodsFor: 'private-utils' stamp: 'SR 8/22/2007 20:02'!
readErrorStream
	error := self readStream: self errorStreamName! !

!OSProcessStreamedRawIO methodsFor: 'private-utils' stamp: 'SR 8/22/2007 20:01'!
readOutputStream
	output := self readStream: self outputStreamName! !

OSProcessIO subclass: #OSProcessStandardIO
	instanceVariableNames: 'model command'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OSProcessIO'!

!OSProcessStandardIO commentStamp: 'SR 8/22/2007 19:29' prior: 0!
REQUIRED PARAMETERS:
- model = the model on which we need some IO
- command = a selector which is executed on the model. The return value should be the command which the model requires (NOT MORE !! -> see subclasses for more parameters)

OUTPUT:
- process = the process resulted from the IO execution!

!OSProcessStandardIO class methodsFor: 'execution' stamp: 'SR 8/22/2007 19:38'!
model: aModel command: aSelector
	^ super new
		model: aModel;
		command: aSelector;
		execute! !

!OSProcessStandardIO methodsFor: 'accessing' stamp: 'SR 7/15/2007 17:00'!
command
	^ command! !

!OSProcessStandardIO methodsFor: 'accessing' stamp: 'SR 8/22/2007 19:29'!
command: aSelector
	self resetCache.
	command := aSelector! !

!OSProcessStandardIO methodsFor: 'private-accessing' stamp: 'SR 7/15/2007 17:05'!
ioCommand
	^ self model perform: self command! !

!OSProcessStandardIO methodsFor: 'accessing' stamp: 'SR 7/15/2007 17:00'!
model
	^ model! !

!OSProcessStandardIO methodsFor: 'accessing' stamp: 'SR 8/22/2007 19:30'!
model: aModel
	self resetCache.
	model := aModel! !

OSProcessStandardIO subclass: #OSProcessStreamedIO
	uses: TOSProcessStreamedIO
	instanceVariableNames: 'error output streamIdentifier streamTarget errorStreamName outputStreamName streamTargetName streamIdentifierName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OSProcessIO'!

!OSProcessStreamedIO commentStamp: 'SR 8/22/2007 21:10' prior: 0!
same as above but it will collect the output and error from the shell by default

POSSIBLE OUTPUT
- error = die error message produced by this process
- output = the output message produced by this process

OPTIONAL PARAMETERS:
- streamIdentifier = a dedicated stream file-ending for this process [default='osprocess']
- streamTarget = a dedicated stream file-name for this process [default=name od the model's class]

INTERNAL CACHES ONLY:
- errorStreamName = the name of the temporary error stream 
- outputStreamName = the name of the temporary output stream
- streamTargetName = the name of the target name
- streamIdentifierName = the name of the target name!

OSProcessStreamedIO class
	uses: TOSProcessStreamedIO classTrait
	instanceVariableNames: ''!

OSProcessStreamedIO subclass: #OSProcessParsedIO
	uses: TOSProcessParsedIO
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OSProcessIO'!

!OSProcessParsedIO commentStamp: 'SR 8/22/2007 21:10' prior: 0!
same as above but it will pass the output from the shell to its parser / back to the model

REQUIRED PARAMETERS:
- parser = a Block taking self as an argument, the model is responsible for the parsing, ....!

OSProcessParsedIO class
	uses: TOSProcessParsedIO classTrait
	instanceVariableNames: ''!

!OSProcessParsedIO methodsFor: 'private' stamp: 'SR 1/7/2009 11:08'!
basicResult
	^ self model perform: self parser with: self! !

!OSProcessParsedIO methodsFor: 'accessing' stamp: 'SR 7/15/2007 17:44'!
parser
	^ parser! !

!OSProcessParsedIO methodsFor: 'accessing' stamp: 'SR 7/15/2007 17:43'!
parser: aSelector
	parser := aSelector! !

!OSProcessStreamedIO methodsFor: 'private' stamp: 'SR 8/22/2007 19:58'!
basicExecute
	process := self primitiveExecute
	" we do not assert this kind of process - instead we have a result and error log "! !

!OSProcessStreamedIO methodsFor: 'private-accessing' stamp: 'SR 8/22/2007 18:34'!
errorStreamName
	^ errorStreamName 
		ifNil: [ errorStreamName := self ioStreamName, '.error' ]! !

!OSProcessStreamedIO methodsFor: 'accessing' stamp: 'SR 8/22/2007 18:54'!
errorString
	^ error! !

!OSProcessStreamedIO methodsFor: 'private-accessing' stamp: 'SR 8/22/2007 20:08'!
ioStreamIdentifier
	^ streamIdentifierName ifNil: [ streamIdentifierName := self streamIdentifier
		ifNotNil: [ self model perform: self streamIdentifier ] 
		ifNil: [ self defaultStreamIdentifier ] ]! !

!OSProcessStreamedIO methodsFor: 'private-accessing' stamp: 'SR 8/22/2007 21:01'!
ioStreamTarget
	^ streamTargetName ifNil: [ streamTargetName :=  self streamTarget
		ifNotNil: [ self model perform: self streamTarget ] 
		ifNil: [ self absoluteUrlFor: self model class name ] ]! !

!OSProcessStreamedIO methodsFor: 'private-accessing' stamp: 'SR 8/22/2007 18:34'!
outputStreamName
	^ outputStreamName 
		ifNil: [ outputStreamName := self ioStreamName, '.output' ]! !

!OSProcessStreamedIO methodsFor: 'accessing' stamp: 'SR 8/22/2007 18:55'!
outputString
	^ output! !

!OSProcessStreamedIO methodsFor: 'private-utils' stamp: 'SR 8/22/2007 18:21'!
readErrorStream
	error := self readStream: self errorStreamName! !

!OSProcessStreamedIO methodsFor: 'private-utils' stamp: 'SR 8/22/2007 18:21'!
readOutputStream
	output := self readStream: self outputStreamName! !

!OSProcessStreamedIO methodsFor: 'private' stamp: 'SR 8/22/2007 19:30'!
resetCache
	super resetCache.
	errorStreamName := nil.
	outputStreamName := nil.
	streamIdentifierName := nil.
	streamTargetName := nil! !

!OSProcessStreamedIO methodsFor: 'accessing' stamp: 'SR 8/22/2007 19:13'!
streamIdentifier
	^ streamIdentifier! !

!OSProcessStreamedIO methodsFor: 'accessing' stamp: 'SR 8/22/2007 19:13'!
streamIdentifier: aSelector
	self resetCache.
	streamIdentifier := aSelector! !

!OSProcessStreamedIO methodsFor: 'accessing' stamp: 'SR 8/22/2007 19:13'!
streamTarget
	^ streamTarget! !

!OSProcessStreamedIO methodsFor: 'accessing' stamp: 'SR 8/22/2007 19:13'!
streamTarget: aSelector
	self resetCache.
	streamTarget := aSelector! !

!ExternalWindowsOSProcess methodsFor: '*OSProcessIO' stamp: 'SR 8/28/2007 17:24'!
executableFor: anObject
	^ (super executableFor: anObject), '.exe'! !

!OSProcess methodsFor: '*OSProcessIO' stamp: 'SR 9/1/2007 09:18'!
executableFor: anObject
	^ anObject command! !

!OSProcess methodsFor: '*OSProcessIO' stamp: 'SR 8/28/2007 17:19'!
executableForCommand: aCommand
	^ aCommand! !
